# -*- coding: utf-8 -*-

# Copyright © Cloud Linux GmbH & Cloud Linux Software, Inc 2010-2021 All Rights Reserved
#
# Licensed under CLOUD LINUX LICENSE AGREEMENT
# http://cloudlinux.com/docs/LICENSE.TXT

import functools
import json
from pathlib import Path
from typing import Dict

from clwpos.cl_wpos_exceptions import WposError
from clwpos.constants import USER_WPOS_DIR
from clwpos.logsetup import setup_logging
from clwpos.utils import home_dir, is_run_under_user
from secureio import write_file_via_tempfile

logger = setup_logging(__name__)


class _CommandProgress:
    """
    Class to represent user's WPOS utility commands progress.
    Should be used as parent class.
    """
    total_stages: int
    progress_file_name: str

    def __init__(self):
        if not is_run_under_user():
            raise WposError('Internal Error: trying to use _CommandProgress class as root')
        self.in_progress = False
        self.completed_stages = 0
        self.progress_file = Path(home_dir(), USER_WPOS_DIR, self.progress_file_name)

    def as_dict(self) -> Dict[str, int]:
        """
        Return number of total and completed stages as dict.
        """
        return {
            'total_stages': self.total_stages,
            'completed_stages': self.completed_stages,
        }

    def start(self) -> None:
        """
        Start tracking the progress of the command.
        """
        self.in_progress = True
        self.completed_stages = 0
        self.progress_file.parent.mkdir(mode=0o700, parents=True, exist_ok=True)
        self._update_progress_file()

    def update(self) -> None:
        """
        Update progress status by incrementing
        the number of completed stages by one.
        """
        if not self.in_progress:
            return
        self.completed_stages += 1
        if self.completed_stages > self.total_stages:
            raise WposError(
                'Internal Error: the number of completed stages '
                'is greater than the total number of stages'
            )
        self._update_progress_file()

    def complete(self) -> None:
        """
        Complete progress, delete file with progress status.
        """
        if not self.in_progress:
            return
        self.in_progress = False
        self.completed_stages = self.total_stages
        if self.progress_file.exists():
            self.progress_file.unlink()

    def _update_progress_file(self) -> None:
        write_file_via_tempfile(
            json.dumps(self.as_dict()), self.progress_file, 0o644)

    @property
    def status(self) -> Dict[str, int]:
        """
        Return current progress status of the command execution,
        getting it from from special file.
        """
        if not self.progress_file.exists():
            return self.as_dict()

        try:
            with open(self.progress_file, 'r') as f:
                status_from_file = json.loads(f.read())
            self.total_stages = int(status_from_file['total_stages'])
            self.completed_stages = int(status_from_file['completed_stages'])
        except Exception as e:
            logger.error('Can\'t parse progress status json: %s', e)

        return self.as_dict()


class EnableCommandProgress(_CommandProgress):
    """
    Class to represent user's WPOS utility "enable" command progress.
    """
    total_stages: int = 8
    progress_file_name: str = '.enable-progress-status'


def track_command_progress(func):
    """
    Decorator for the WPOS utility commands
    we want to track the progress for.
    """
    @functools.wraps(func)
    def wrapper(self, *args, **kwargs):
        self.command_progress = EnableCommandProgress()
        self.command_progress.start()
        try:
            return func(self, *args, **kwargs)
        finally:
            self.command_progress.complete()
    return wrapper


def update_command_progress(func):
    """
    Decorator for the WPOS utility methods after which
    the counter of completed operations must be updated.
    """
    @functools.wraps(func)
    def wrapper(self, *args, **kwargs):
        result = func(self, *args, **kwargs)
        if getattr(self, 'command_progress') is not None:
            self.command_progress.update()
        return result
    return wrapper
